@@ -1,4 +1,4 @@
-/*  Copyright (c) 2003-2013 Xfce Development Team
+/*  Copyright (c) 2003-2014 Xfce Development Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,15 +31,21 @@
 #define LABEL_SLEEP (3)       /* sleep time in seconds */
 #define LABEL_SLEEP_LONG (6)  /* sleep time in seconds for FADE_NONE */
 #define LABEL_SPEED (25)      /* animation speed, delay in ms */
+#define LABEL_PADDING (3)     /* padding to left/right or top/bottom of label */
 
 
 static void gtk_scrollbox_finalize(GObject *object);
 
-static void gtk_scrollbox_size_request(GtkWidget *widget,
-                                       GtkRequisition *requisition);
+static void gtk_scrollbox_get_preferred_height (GtkWidget *widget,
+                                                gint      *minimal_height,
+                                                gint      *natural_height);
 
-static gboolean gtk_scrollbox_expose_event(GtkWidget *widget,
-                                           GdkEventExpose *event);
+static void gtk_scrollbox_get_preferred_width (GtkWidget *widget,
+                                               gint      *minimal_width,
+                                               gint      *natural_width);
+
+static gboolean gtk_scrollbox_draw_event(GtkWidget *widget,
+                                         cairo_t   *cr);
 
 static gboolean gtk_scrollbox_control_loop(gpointer user_data);
 
@@ -56,15 +62,16 @@ gtk_scrollbox_class_init(GtkScrollboxClass *klass)
     gobject_class->finalize = gtk_scrollbox_finalize;
 
     widget_class = GTK_WIDGET_CLASS(klass);
-    widget_class->size_request = gtk_scrollbox_size_request;
-    widget_class->expose_event = gtk_scrollbox_expose_event;
+    widget_class->get_preferred_width = gtk_scrollbox_get_preferred_width;
+    widget_class->get_preferred_height = gtk_scrollbox_get_preferred_height;
+    widget_class->draw = gtk_scrollbox_draw_event;
 }
 
 
 static void
 gtk_scrollbox_init(GtkScrollbox *self)
 {
-    GTK_WIDGET_SET_FLAGS(self, GTK_NO_WINDOW);
+    gtk_widget_set_has_window (GTK_WIDGET (self), FALSE);
 
     self->labels = NULL;
     self->labels_new = NULL;
@@ -170,20 +177,55 @@ gtk_scrollbox_size_request(GtkWidget *widget,
 }
 
 
+static void
+gtk_scrollbox_get_preferred_height (GtkWidget *widget,
+                                    gint      *minimal_height,
+                                    gint      *natural_height)
+{
+    GtkScrollbox *self = GTK_SCROLLBOX(widget);
+    GtkRequisition requisition;
+
+    gtk_scrollbox_size_request (widget, &requisition);
+
+    if (self->orientation == GTK_ORIENTATION_VERTICAL)
+        *minimal_height = *natural_height = requisition.height + (LABEL_PADDING * 2);
+    else
+        *minimal_height = *natural_height = requisition.height;
+}
+
+
+static void
+gtk_scrollbox_get_preferred_width (GtkWidget *widget,
+                                   gint      *minimal_width,
+                                   gint      *natural_width)
+{
+    GtkScrollbox *self = GTK_SCROLLBOX(widget);
+    GtkRequisition requisition;
+
+    gtk_scrollbox_size_request (widget, &requisition);
+
+    if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
+        *minimal_width = *natural_width = requisition.width + (LABEL_PADDING * 2);
+    else
+        *minimal_width = *natural_width = requisition.width;
+}
+
+
 static gboolean
-gtk_scrollbox_expose_event(GtkWidget *widget,
-                           GdkEventExpose *event)
+gtk_scrollbox_draw_event(GtkWidget *widget,
+                         cairo_t   *cr)
 {
     GtkScrollbox *self = GTK_SCROLLBOX(widget);
     PangoLayout *layout;
-    gint width, height;
+    gint height, width;
     PangoRectangle logical_rect;
     gboolean result = FALSE;
     PangoMatrix matrix = PANGO_MATRIX_INIT;
+    GtkAllocation allocation;
 
-    if (GTK_WIDGET_CLASS(gtk_scrollbox_parent_class)->expose_event != NULL)
+    if (GTK_WIDGET_CLASS(gtk_scrollbox_parent_class)->draw != NULL)
         result = GTK_WIDGET_CLASS
-            (gtk_scrollbox_parent_class)->expose_event(widget, event);
+            (gtk_scrollbox_parent_class)->draw(widget, cr);
 
     if (self->active != NULL) {
         layout = PANGO_LAYOUT(self->active->data);
@@ -193,31 +235,25 @@ gtk_scrollbox_expose_event(GtkWidget *widget,
         pango_context_set_matrix(pango_layout_get_context(layout), &matrix);
         pango_layout_get_extents(layout, NULL, &logical_rect);
 
+        gtk_widget_get_allocation (GTK_WIDGET (widget), &allocation);
+
         if (self->orientation == GTK_ORIENTATION_HORIZONTAL) {
-            width = widget->allocation.x
-                + (widget->allocation.width
-                   - PANGO_PIXELS(logical_rect.width)) / 2;
-            height = widget->allocation.y
-                + (widget->allocation.height
-                   - PANGO_PIXELS(logical_rect.height)) / 2
+            width = LABEL_PADDING;
+            height = (allocation.height - PANGO_PIXELS(logical_rect.height)) / 2
                 + (self->fade == FADE_IN || self->fade == FADE_OUT
                    ? self->offset : 0);
         } else {
-            width = widget->allocation.x
-                + (widget->allocation.width
-                   - PANGO_PIXELS(logical_rect.height)) / 2
+            height = LABEL_PADDING;
+            width = (allocation.width + PANGO_PIXELS(logical_rect.height)) / 2
                 + (self->fade == FADE_IN || self->fade == FADE_OUT
                    ? self->offset : 0);
-            height = widget->allocation.y
-                + (widget->allocation.height
-                   - PANGO_PIXELS(logical_rect.width)) / 2;
         }
 
-        gtk_paint_layout(widget->style,
-                         widget->window,
-                         GTK_WIDGET_STATE(widget), TRUE,
-                         &event->area, widget,
-                         "GtkScrollbox", width, height, layout);
+        gtk_render_layout (gtk_widget_get_style_context (GTK_WIDGET (widget)),
+                           cr,
+                           width,
+                           height,
+                           layout);
     }
     return result;
 }
@@ -304,6 +340,7 @@ static gboolean
 gtk_scrollbox_fade_out(gpointer user_data)
 {
     GtkScrollbox *self = GTK_SCROLLBOX(user_data);
+    GtkAllocation allocation;
 
     /* increase counter */
     if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
@@ -312,11 +349,12 @@ gtk_scrollbox_fade_out(gpointer user_data)
         self->offset--;
 
     gtk_widget_queue_draw(GTK_WIDGET(self));
+    gtk_widget_get_allocation (GTK_WIDGET (self), &allocation);
 
     if ((self->orientation == GTK_ORIENTATION_HORIZONTAL &&
-         self->offset < GTK_WIDGET(self)->allocation.height) ||
+         self->offset < allocation.height) ||
         (self->orientation == GTK_ORIENTATION_VERTICAL &&
-         self->offset > 0 - GTK_WIDGET(self)->allocation.width))
+         self->offset > 0 - allocation.width))
         return TRUE;
 
     (void) gtk_scrollbox_control_loop(self);
@@ -332,6 +370,7 @@ static gboolean
 gtk_scrollbox_control_loop(gpointer user_data)
 {
     GtkScrollbox *self = GTK_SCROLLBOX(user_data);
+    GtkAllocation allocation;
 
     if (self->timeout_id != 0) {
         g_source_remove(self->timeout_id);
@@ -344,7 +383,10 @@ gtk_scrollbox_control_loop(gpointer user_data)
         self->fade = FADE_SLEEP;
         break;
     case FADE_OUT:
-        self->fade = FADE_IN;
+        if (self->animate)
+            self->fade = FADE_IN;
+        else
+            self->fade = FADE_NONE;
         gtk_scrollbox_swap_labels(self);
         break;
     case FADE_NONE:
@@ -363,14 +405,16 @@ gtk_scrollbox_control_loop(gpointer user_data)
         break;
     }
 
+    gtk_widget_get_allocation (GTK_WIDGET (self), &allocation);
+
     /* now perform the next action */
     switch(self->fade) {
     case FADE_IN:
         if (self->labels_len > 1) {
             if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
-                self->offset = GTK_WIDGET(self)->allocation.height;
+                self->offset = allocation.height;
             else
-                self->offset = 0 - GTK_WIDGET(self)->allocation.width;
+                self->offset = 0 - allocation.width;
         } else
             self->offset = 0;
         self->timeout_id = g_timeout_add(LABEL_SPEED,
@@ -390,9 +434,9 @@ gtk_scrollbox_control_loop(gpointer user_data)
         break;
     case FADE_NONE:
         if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
-            self->offset = GTK_WIDGET(self)->allocation.height;
+            self->offset = allocation.height;
         else
-            self->offset = GTK_WIDGET(self)->allocation.width;
+            self->offset = allocation.width;
         self->timeout_id = g_timeout_add_seconds(LABEL_SLEEP_LONG,
                                                  gtk_scrollbox_control_loop,
                                                  self);
@@ -473,7 +517,7 @@ gtk_scrollbox_set_visible(GtkScrollbox *self,
             (void) gtk_scrollbox_control_loop(self);
         } else {
             /* update immediately if there's only one or no label,
-             typically this is the case at startup */
+               typically this is the case at startup */
             if (self->active == NULL || self->labels_len <= 1)
                 (void) gtk_scrollbox_control_loop(self);
         }
@@ -532,15 +576,20 @@ gtk_scrollbox_set_fontname(GtkScrollbox *self,
 
 void
 gtk_scrollbox_set_color(GtkScrollbox *self,
-                        const GdkColor color)
+                        const GdkRGBA color)
 {
     PangoAttribute *pattr;
+    guint16 red, green, blue;
 
     g_return_if_fail(GTK_IS_SCROLLBOX(self));
 
-    pattr = pango_attr_foreground_new(color.red,
-                                      color.green,
-                                      color.blue);
+    red = (guint16) (color.red * 65535);
+    green = (guint16) (color.green * 65535);
+    blue = (guint16) (color.blue * 65535);
+
+    pattr = pango_attr_foreground_new(red,
+                                      green,
+                                      blue);
     pango_attr_list_change(self->pattr_list, pattr);
 
     /* update all labels */
